
:- ensure_loaded( library(ordsets) ).   % list_to_ord_set/2.
:- ensure_loaded( '../../src/lib/read_terms' ). % read_terms/2.
:- requires( kv_decompose/3 ).

% upsh ~/islp/mcmcms/auxil/data/data_terms_to_possible_values.pl van_steensel_fst1000_data.pl van_steensel_fst1000_psvls.pl
% cd ~/nki/proj/dat
% upsh ~/islp/mcmcms/auxil/data/data_terms_to_possible_values.pl fa_t0_wide_15.pl

%
% data_terms_to_possible_values( Data, Output ) :-
%    interface predicate ot data_terms_to_possible_values/4.
%    Data could be a file or list of terms, in the former the data are read in
%    to a list- so files should only contain the data terms.
%    Originally for data/n terems only; now data/n-Tms terems also acceptable.
%    CAUTION. the - syntax is the wrong one for data/2.... 
data_terms_to_possible_values( DataSrc, Output ) :-
     ( is_stream(Output)  ->
          Out = Output
          ;
          open( Output, write, Out )
     ),
     ( DataSrc = [H|_] -> 
          Data = DataSrc
          ;
          read_terms( DataSrc, Data ),
          Data = [H|_]
     ),
     ( H = Dtm-_Tms ->
          functor( Dtm, _, Arity ),
          kv_decompose( Data, DataOnly, _TmsList )
          ;
          functor( H, _, Arity ),
          DataOnly = Data
     ),
     data_terms_to_possible_values( Arity, 1, DataOnly, Out, DmSizes ),
     nl( Out ),
     findall( VarId, (member(VarId-Sz,DmSizes),portray_clause(Out,domain_size(VarId,Sz))), _Vs ),
     ( is_stream(Output) ->
          true
          ;
          close( Out )
     ).
          

% data_terms_to_possible_values( Arity, I, Data, Out, DmSzPairs ) :-
% Report to stream Out all possible values for each of the Ith Args 
% where I =< Arity. Each reported term has the form
% possible_values( I, IValuesInData ).
%
data_terms_to_possible_values( 0, _N, _Data, _Out, [] ) :- !.
data_terms_to_possible_values( N, I, Data, Out, [I-Sz|T] ) :-
     select_arg( Data, 1, First, Rest ),
     list_to_ord_set( First, Set ),
     portray_clause( Out, possible_values(I,Set) ),
     length( Set, Sz ),
     Nxt is N - 1,
     J is I + 1,
     data_terms_to_possible_values( Nxt, J, Rest, Out, T ).

select_arg( [], _A, [], [] ).
select_arg( [H|T], A, [Ath|Aths], [HmAth|TmAths] ) :-
     % arg( A, H, Ath ),
     H =.. [F|Args],
     nth_rem( A, Args, Ath, HmArgs ),
     HmAth =.. [F|HmArgs],
     select_arg( T, A, Aths, TmAths ).

nth_rem( 1, [H|T], H, T ) :- !.
nth_rem( N, [H|T], Nth, [H|R] ) :-
     Nxt is N - 1,
     nth_rem( Nxt, T, Nth, R ).

