/* 
     discretise the args of a list of terms.

     discretise( [1-3], [r(1,2),r(2,3),r(3,4),r(4,5),r(5,6),r(6,7)], New ).
     discretise( [1-3], [r(0.1,2),r(0.2,3),r(0.3,4),r(0.4,5),r(0.5,6),r(0.6,7)], New ).
     */

:- requires( nth_replace/5 ).

/*
     discretise( Arg#-NumberOfBins, InTerms, OutTerms ).
     Out Terms have same name and arity with InTerms.
*/

discretise( [], Terms, Terms ).
discretise( [A-Bins|T], Terms, NewTerms ) :-
     findall( G, (member(S,Terms),arg(A,S,G)), Gs ),
     min_list( Gs, Mn ),
     max_list( Gs, Mx ),
     Step is (Mx - Mn) / Bins,
     write( d(A,Bins,Mn,Mx,Step) ), nl,
     % ( A =:= 9 -> trace; true ),
     findall( MaxVal, (between(1,Bins,I),MaxVal is Mn  + (I*Step)), Maxi ),
     break_bins( Gs, Maxi, NewGs ),
     replace_ith_arg( Terms, NewGs, A, NxTerms ), 
     discretise( T, NxTerms, NewTerms ).

break_bins( [], _Maxi, [] ).
break_bins( [H|T], Maxi, [NewH|NewT] ) :-
     break_point( Maxi, 1, H, NewH ),
     break_bins( T, Maxi, NewT ).

% break_point( [], I, H ) :- fail.
break_point( [], I, _H, I ).   % float arithmetics some times make the last limit > last obs
break_point( [H|T], I, P, New ) :-
     H < P,
     !,
     NxI is I + 1,
     break_point( T, NxI, P, New ).
break_point( [_H|_T], I, _P, I ).

replace_ith_arg( [], [], _N, [] ).
replace_ith_arg( [H|T], [Hg|TGs], N, [NewH|NewT] ) :-
     H =.. [Name|Args],
     nth_replace( N, Args, Hg, _Old, NewArgs ),
     NewH =.. [Name|NewArgs],
     replace_ith_arg( T, TGs, N, NewT ).
