
:- requires( opts_select_infiles/3 ).
:- requires( append/3 ).
:- requires( memberchk/2 ).
:- requires( has_extension/3 ).
:- requires( option_required/2 ).
:- requires( kv_compose/3 ).
:- requires( kv_v_onto_n_bins/6 ).

% lupsh percent_positives_against_bins pos=7 pstv=n ~/islp/sn/eval_assess_seldi_set3.csv
percent_positives_against_bins( Opts ) :-
	opts_select_infiles( Opts, Fs, OthOpts ),
	percent_positives_again_bins_defaults( Defs ),
	append( Defs, OthOpts, All ),
	memberchk( bins(Bins), All ),
	option_required( pos(Pos), All ),
	( memberchk(pstv(Pstv),OthOpts) -> true; true ),
	pcent_pstvs_vs_bins( Fs, Pos, Bins, Pstv, All ).

percent_positives_again_bins_defaults( [bins(10)] ).

pcent_pstvs_vs_bins( [], _Pos, _Bins, _Pstv, _Opts ).
pcent_pstvs_vs_bins( [H|T], Pos, Bins, Pstv, Opts ) :-
	open( H, read, In ),
	read( In, FstDTerm ),
	close( In ),
	functor( FstDTerm, data, LstPos ),
	datafile_read_positions( H, [Pos,LstPos], KVPsts ),
	% datafile_read_positions( H, [Pos,LstPos], [PcPsts,ClPsts] ),
	% kv_compose( PcPsts, ClPsts, KVPsts ),
	kv_v_onto_n_bins( KVPsts, Bins, Increments, Max, _Min, TheBins ),
	( var(Pstv) -> 
		arg( LstPos, FstDTerm, HPstv )
		;
		HPstv = Pstv
	),
	count_positive_in_bins( TheBins, HPstv, Pstvs, Ngtvs ),
	has_extension( H, Base, _Ext ),
	number_codes( Pos, PosCs ),
	append( PosCs, "_vsb", PosVsbCs ),
	number_codes( Bins, BinsCs ),
	append( PosVsbCs, BinsCs, CmnSfxCs ),
	append( ".pcent_", CmnSfxCs, SfxPcentCs ),
	append( ".abs_", CmnSfxCs, SfxAbsCs ),
	atom_codes( SfxPc, SfxPcentCs ),
	atom_codes( SfxAb, SfxAbsCs ),
	atom_concat( Base, SfxPc, FilePc ),
	atom_concat( Base, SfxAb, FileAb ),
	open( FilePc, write, OutPc ),
	open( FileAb, write, OutAb ),
	write_kv_for_plots( Increments, Pstvs, Ngtvs, Max, OutPc, OutAb ),
	close( OutPc ),
	close( OutAb ),
	pcent_pstvs_vs_bins( T, Pos, Bins, Pstv, Opts ).

write_kv_for_plots( [], [Hp], [Hn], Max, OutPc, OutAb ) :-
	write( OutAb, Max ), write( OutAb, '  ' ), write( OutAb, Hp ),
	nl( OutAb ),
	Pc is (Hp/Hn) * 100,
	write( OutPc, Max ), write( OutPc, '  ' ), write( OutPc, Pc ),
	nl( OutPc ).
write_kv_for_plots( [Hi|Tis], [Hp|Tps], [Hn|Tns], Max, OutPc, OutAb ) :-
	write( OutAb, Hi ), write( OutAb, '  ' ), write( OutAb, Hp ),
	nl( OutAb ),
	Pc is (Hp/Hn) * 100,
	write( OutPc, Hi ), write( OutPc, '  ' ), write( OutPc, Pc ),
	nl( OutPc ),
	write_kv_for_plots( Tis, Tps, Tns, Max, OutPc, OutAb ).

datafile_read_positions( File, Positions, ReadPsts ) :-
	open( File, read, In ),
	read( In, Fst ),
	datastream_read_positions( Fst, Positions, In, ReadPsts ),
	close( In ).

datastream_read_positions( end_of_file, _Positions, _In, ReadPsts ) :-
	!,
	ReadPsts = [].
datastream_read_positions( Term, Positions, In, [K-V|TReadPsts] ) :-
	args_of_term( Positions, Term, [V,K] ),
	% ReadPsts = [HPsts|TReadPsts],
	read( In, NxTerm ),
	datastream_read_positions( NxTerm, Positions, In, TReadPsts ).

args_of_term( [], _Term, [] ).
args_of_term( [H|T], Term, [Harg|Targs] ) :-
	arg( H, Term, Harg ),
	args_of_term( T, Term, Targs ).

count_positive_in_bins( [], _Pstv, [], [] ).
count_positive_in_bins( [H|T], Pstv, [HPstvs|TPstvs], [HNgtvs|TNgtvs] ) :-
	count_positive_in_bin( H, Pstv, 0, 0, HPstvs, HNgtvs ),
	count_positive_in_bins( T, Pstv, TPstvs, TNgtvs ).

count_positive_in_bin( [], _Pstv, Pstvs, Ngtvs, Pstvs, Ngtvs ).
count_positive_in_bin( [K-_V|T], Pstv, AcPstvs, AcNgtvs, Pstvs, Ngtvs ) :-
	( K == Pstv -> 
		NxPstvs is AcPstvs + 1, 
		NxNgtvs is AcNgtvs
		;
		NxPstvs is AcPstvs,
		NxNgtvs is AcNgtvs + 1 
	),
	count_positive_in_bin( T, Pstv, NxPstvs, NxNgtvs, Pstvs, Ngtvs ).
