:- ensure_loaded( '../src/init_lib'  ).
:- ensure_loaded( '../src/all_dynamic'  ).
:- ensure_loaded( '../src/ad_to_slp'  ).
:- ensure_loaded( library(lists) ).				% memberchk/2.

:- ensure_loaded( library(kvs_add_v_to_matching_k) ).	% /4.
:- ensure_loaded( library(delete_file_if) ).			% /1.
:- pl( swi(_), true, ensure_loaded(library(flatten)) ).% /2.
:- ensure_loaded( library(slp_in_options) ).			% /2.
:- ensure_loaded( library(tmp_in_options) ).			% /3.
:- ensure_loaded( library(pl_to_slp_call) ).			% /2, _args/3.

% Opts: 
%	slp/1, samples/1, dot_ith_intervals/1, precall/1, call/1, post/1, tmp/1, stem/1.
%
n_samples( Opts, Tmp, Freqs ) :-
	n_samples_defaults( Defs ),
	append( Opts, Defs, All ), 
	slp_in_options( All, Slp ),
	memberchk( samples(Samples), All ),
	memberchk( dot_ith_intervals(Intervals), All ),
	Dot is Samples // Intervals,
	tmp_in_options( Opts, All, Tmp ),
	all_dynamic( Slp ),
	memberchk( ad_to_slp(AdSlp), All ),
	append( AdSlp, [rm(false),tmp(Tmp)], AdSlpSend ),
	ad_to_slp( AdSlpSend ),
	memberchk( call(PlCall-PrvObs),All), 
	pl_to_slp_call_args( PlCall, SlpCall, ConvArgs ),
	( memberchk(recpath(true),All) -> 
		memberchk(path(Path),ConvArgs),
		Obs=PrvObs/Path
		;
		Obs = PrvObs
	),
	memberchk( precall(PreCall), All ),
	call( PreCall ),
	memberchk( post(Post), All ),
	memberchk( res(ResForm), All ),
	( ResForm == nat ->
		gen_n_samples_nat( Samples, Dot, SlpCall/Obs, Post, Freqs )
		;
		gen_n_samples_acc( Samples, Dot, SlpCall/Obs, Post, [], Freqs )
	).

% Stem should always be present.
n_samples_defaults( Defs ) :-
	Defs = [stem('split-2'),samples(100),dot_ith_intervals(10),
			call(cart(NofLs)-NofLs),slp('split.slp'),precall(true),
			post(identical),res(frq),ad_to_slp([])].
	
gen_n_samples_nat( 0, _Dot, _Call/_Obs, _Post, [] ) :- !, nl.
gen_n_samples_nat( N, Dot, Call/Obs, Post, [H|T] ) :-
	copy_term( Call/Obs, Fcall/Fobs ),
	( call( slp:Fcall ) ->   % how about slp:call(Fcal)
		% check it is ground ?
		PostCall =.. [Post,Fobs,H],
		call( PostCall )
		;
		H = failure
	),
	NxN is N - 1,
	( 0 =:= ((N-1) mod Dot) -> write( '.' ),flush_output ; true ),
	gen_n_samples_nat( NxN, Dot, Call/Obs, Post, T ).

gen_n_samples_acc( 0, _DotInterval, _Call/_Obs, _Post, YsFrq, YsFrq ) :- !, nl.
gen_n_samples_acc( N, DotInterval, Call/Obs, Post, Acc, YsFrq ) :-
	copy_term( Call/Obs, Fcall/Fobs ),
	( call( slp:Fcall ) ->   % how about slp:call(Fcal)
		% check it is ground ?
		PostCall =.. [Post,Fobs,Key],
		call(PostCall)
		;
		% write( user_error, caught_error ), nl(user_error), halt
		Key = failure
	),
	kvs_add_v_to_matching_k( Acc, Key, 1, NxAcc ),
	NxN is N - 1,
	( 0 =:= ((N-1) mod DotInterval) -> write( '.' ),flush_output ; true ),
	gen_n_samples_acc( NxN, DotInterval, Call/Obs, Post, NxAcc, YsFrq ).

identical( A, A ).
