:- ensure_loaded( '../../src/init_lib' ).

:- ensure_loaded( library(lists) ).		          % nth/3, append/3.
:- ensure_loaded( library(charsio) ).		          %.

:- ensure_loaded( library(mcmcms_top_dir) ).		% bb_puts it

:- requires( opts_select_infiles/3 ).
:- requires( werr/1 ).
:- requires( elm/1 ).
:- requires( to_number/1 ).

:- elm( 'auxil/data_calls_from_stats_file' ).   % /2.
:- elm( 'src/model_type_calls' ).

:- ensure_loaded( library(lists) ).          % memberchk/2.

:- requires( read_terms/2 ).
:- requires( werr/2 ).

gvz_nth_fterm( Opts ) :-
     % listing( elm/1 ),
	opts_select_infiles( Opts, [File|_Fs], OthOpts ),
	atom_concat( BareFile, '.fterms', File ),
     nl, nl,
     data_calls_from_stats_file( BareFile, ModType ),
     modtype_loads( ModType, RedModType, Arg ),
	gvz_nth_fterm_defaults( Defs ),
	append( OthOpts, Defs, AllOpts ),
	memberchk( from(PrvFrom), AllOpts ),
     to_number( PrvFrom, From ),
	memberchk( for(PrvFor), AllOpts ),
     to_number( PrvFor, For ),
	nl,
     werr( ['Using file: ',File], info ),
     werr( ['Using from: ',From], info ),
     werr( ['Using  for: ',For ], info ),
     nl, nl,
	From > 0,
	open( File, read, In ),
	gvz_display_options( AllOpts, DOpts ),
	gvz_nth_fterm_on_stream( From, For, In, RedModType, Arg, DOpts ),
	close( In ).

gvz_display_options( AllOpts, DOpts ) :-
	( select(colour(Clr), AllOpts, RemOpts) ->
		( Clr = bnw -> 
			DOpts = [colour(Clr),node_style(solid)|RemOpts]
			;
			DOpts = [colour(Clr)|RemOpts]
		)
		;
		DOpts = AllOpts
	).

gvz_nth_fterm_defaults( Defs ) :-
	Defs = [ from(1), for(1), exec(gv) ].
     % change gv to xv for pngs....

gvz_nth_fterm_on_stream( 1, For, In, Type, Arg, DOpts ) :-
	!,
	gvx_next_n( For, In, Type, Arg, DOpts ).
gvz_nth_fterm_on_stream( N, For, In, Type, Arg, DOpts ) :-
	read( In, _FTerm ),
	NxN is N - 1,
	gvz_nth_fterm_on_stream( NxN, For, In, Type, Arg, DOpts ).

gvx_next_n( For, In, Type, Arg, DOpts ) :-
	For > 0,
	!,
	gvx_model( Type, Arg, In, DOpts ),
	NxFor is For - 1,
	gvx_next_n( NxFor, In, Type, Arg ).
gvx_next_n( _, _In, _Type, _Arg ).

gvx_model( cart_red, Ids, In, DOpts ) :-
	!,
	read( In, st(Tms,LL,PrvMis,RedM) ),
     ( atomic(PrvMis) -> 
          Mis = PrvMis, 
          Tit = Tit1
          ;
          PrvMis = [Mis|TMis],
          write_list_to_atoms_list( TMis, Tits ),
          Tit = [Tit1|Tits]
     ),
	write_to_chars( vst(Tms):mclsf(Mis), Codes ),
	atom_codes( Tit1, Codes ),
	( red_cart_to_full( RedM, 0, Ids, FullM ) -> true; FullM = RedM ),
	% display_model( FullM, [title(Atm),lhood(LL),exec('/local/d1p1/nicos/kong/nicos_from_d0p6/tara/local/share/graphviz/demo/doted')|DOpts] ).
     memberchk( exec(Exec), DOpts ),
	display_model( FullM, [title(Tit),lhood(LL),exec(Exec)|DOpts] ).
     % change the gv to xv for pngs....

write_list_to_atoms_list( [], [] ).
write_list_to_atoms_list( [H|T], [Htit|TTits] ) :-
     write_to_chars( H, HtitCs ),
     atom_codes( Htit, HtitCs ),
     write_list_to_atoms_list( T, TTits ).

modtype_loads( cart, cart, null ) :-
	elm( 'carts/src/lhood' ),
     elm( 'carts/src/display_model' ),
	elm( 'carts/auxil/cart_misclassifications' ).
modtype_loads( cart_red(_), cart_red, Ids ) :-
	elm( 'carts/auxil/red_cart_to_full' ),
	once( modtype_loads(cart,_,_) ),
	bb_get( idsd_put, t(_NofF,_DataSet,_Splits,Ids) ).
modtype_loads( ped, ped, null ) :-
	elm( 'pedigree/src/ped_likelihood' ).

