:- ensure_loaded( '../kyph_data' ).
:- ensure_loaded( '../../src/cart_llhood' ).
:- ensure_loaded( '../../src/display_model' ).

r_best_skeleton( cart(3,8,leaf,cart(3,14,leaf,cart(1,55,leaf,cart(1,111,leaf,leaf)))) ).

r_best : - r_best( [] ).

r_best( [] ) :-
	kyph_data( _, _ ),
	findall( [A,B,C]-Class, data(A,B,C,Class), DataPairs ),
	data_pairs_to_skeleton_data( DataPairs, 1, Data ),
	r_best_skeleton( Skel ),
	decision_cart_to_cart( Skel, Data, 0, Cart ),
	cart_llhood( Cart, LLhood ),
	display_model( 'R_learned', Cart, LLhood ).

chipman_best_cart( Cart ) :-
	bcw_data( _, _ ),
	findall( [A,B,C,D,E,F,G,H,I]-Class, data(_,A,B,C,D,E,F,G,H,I,Class), DataPairs ),
	data_pairs_to_skeleton_data( DataPairs, 1, Data ),
	chipman_best_skeleton( Skel ),
	decision_cart_to_cart( Skel, Data, 0, Cart ).

decision_cart_to_cart( cart(F,Val,L,R), Data, Depth, cart(F,Val,Cl,Cr) ) :-
	split_data_on_feature( Data, F, Val, LeftD, RightD ),
	NxDepth is Depth + 1,
	decision_cart_to_cart( L, LeftD, NxDepth, Cl ),
	decision_cart_to_cart( R, RightD, NxDepth, Cr ).
decision_cart_to_cart( leaf, Data, Depth, leaf(Depth,Ids) ) :-
	skeleton_data_to_ids( Data, Ids ).

skeleton_data_to_ids( [], [] ).
skeleton_data_to_ids( [d(_FList,_Class,Id)|T], [Id|Ids] ) :-
	skeleton_data_to_ids( T, Ids ).

data_pairs_to_skeleton_data( [], _N, [] ).
data_pairs_to_skeleton_data( [Flist-Cat|T], N, [d(Flist,Cat,N)|R] ) :-
	NxN is N + 1,
	data_pairs_to_skeleton_data( T, NxN, R ).

split_data_on_feature( [], _F, _Val, [], [] ).
split_data_on_feature( [d(Flist,Cat,Id)|T], F, Val, LeftD, RightD ) :-
	nth( F, Flist, ListVal ),
	( ListVal > Val ->
		RightD = [d(Flist,Cat,Id)|TRight], LeftD = TLeft
		;
		LeftD = [d(Flist,Cat,Id)|TLeft], RightD = TRight
	),
	split_data_on_feature( T, F, Val, TLeft, TRight ).
