
% :- source.

:- ensure_loaded( library(lists) ).				% select/3.
:- ensure_loaded( library(ordsets) ).				% ord_intersection/4.

:- ensure_loaded( '../../src/init_lib' ).
:- ensure_loaded( library(select_n_uniformly) ).		% /5.
:- ensure_loaded( library(kv_decompose_on_max_v) ).	% /3.
:- ensure_loaded( library(kv_compose_fixed_v) ).		% /3.
:- ensure_loaded( library(kv_sum_vs) ).				% /2.
:- ensure_loaded( library(kvs_incr_v_on_keys) ).		% /4.
:- ensure_loaded( library(mcmcms_top_dir) ).			% bb_puts it. 
:- ensure_loaded( library(elm) ).					% /1.
% :- elm( 'carts/slps/data_to_split_root' ).			% /3.

%% :- multifile( null_value/1 ).
%% :- dynamic( null_value/1 ).

% assumes:
%   has_category( Id, Kat ).
%   bb_get( all_categories, Kats ).
%
cart_classifies( Cart, TrainData, Data, Distros, ClsfPrs ) :-
	cart_classifies( Cart, TrainData, Data, bayes, Distros, ClsfPrs ).

cart_classifies( Cart, TrainData, Data, Adj, Distros, ClsfPrs ) :-
	Data = [H|_T],
	functor( H, _Data, Arity ),
	cart_classifies( Cart, TrainData, Data, Arity, Adj, Distros, ClsfPrs ).

% cart_classifies( +Cart, +TData, +Data, +TrCls, +ClArg, +Adj, -Distros, -ClsfPrs ) :-
% Distros is a list of distributions, one for each Datum in Data 
% and ClsPrs is a corresponding list of TrueClass-PrbInDistro pairs. 
% Last argument in Datum is assumed to be the true class.
% Distro is the leaf distribution for the leaf in which the Datum belongs
% within Cart. A distro is a key sorted list of Class-Prb pairs.
% TData, is the training data.
% Adj, provides a handle for adjusting the answer distribution. 
% Currently, its value can be set to none or *bayes*.
%
/*  Old Stuff :
cart_classifies( Cart, TrainData, Data, ClArg, Adj, Distros, ClsfPrs ) :-
	data_to_split_root( TrainData, _Splits, Ids ),
	cart_classifies_no_split( Cart, Ids, Data, ClArg, Adj, Distros, ClsfPrs ).
     */

% old name: cart_classifies_no_split( Cart, Ids, Data, ClArg, Adj, Distros, ClsfPrs ) :-
cart_classifies( Cart, Ids, Data, ClArg, Adj, Distros, ClsfPrs ) :-
	cart_to_classifier( Cart, Ids, Adj, Classifier ),
     % write( done_classifier(Classifier) ), nl,
	list_classification( Data, Classifier, ClArg, Distros, ClsfPrs ).

list_classification( [], _Classifier, _ClArg, [], [] ).
list_classification( [H|T], Classifier, ClArg, [HDst|TDsts], [HPr|TPrs] ) :-
     %% flush_output( user_error ),
     % write( going_classification ), nl,
	classification( Classifier, H, ClArg, HDst, HPr ),
     % cohese_null_multi_classes( HDstHash, HPrHash, HDst, HPr ),
	list_classification( T, Classifier, ClArg, TDsts, TPrs ).

classification( cart(F,Fvl,L,R), Datum, ClArg, Distro, ClsfPr ) :-
     % write( cart(F,Fvl) ), nl,
     ( Datum = data( DList, _Cat ) -> 
          nth( F, DList, Arg )
          ;
	     arg( F, Datum, Arg )
     ),
     % ( null_value(Arg) ->
	     % classification( MidCart, Datum, ClArg, MidDistro, MidClsfPr ),
          % Distro = MidDistro # MainDistro,
          % ClsfPr = MidClsfPr # MainClsfPr
          % ;
          % Distro = MainDistro,
          % ClsfPr = MainClsfPr,
          /*
          compare( Op, Arg, Fvl ),
          ( Op == (>) ->
               NxCart = R
               ;
		     NxCart = L 
          ),
          */
          ( Arg > Fvl  -> 
               NxCart = R
               ;
		     NxCart = L 
          ),
	% ),
     % write( arg(Arg) ), nl,
	classification( NxCart, Datum, ClArg, Distro, ClsfPr ).
classification( leaf(Distro), Datum, ClArg, Distro, DtmClass-(Prb:Maxml) ) :-
	arg( ClArg, Datum, DtmClass ),
	Distro = [Hk-Hp|T],
	distribution_has_maximal_kategory( [Hk-Hp|T], Hk, Hp, DtmClass, Prb, Maxml ),
	( var(Prb) -> throw( an_error ) ; true ).

/*
cohese_null_multi_classes( D1#Ds, Pr1#Prs, CoDs, CoPrs ) :-
     !,
     cohese_null_multi_distros( Ds, D1, 1, CoDs ),
     here.
cohese_null_multi_classes( Distro, Pair, Distro, Pair ).

cohese_null_multi_distros( [], AccD, I, CoDs ) :-
cohese_null_multi_distros( [D|Ds], AccD, I, CoDs ) :-
     here,
     add_lists( AccD, D, NxtD ),
     NxI is I + 1,
     cohese_null_multi_distros( Ds, NxtD, NxI, CoDs ).
     */

distribution_has_maximal_kategory( [], _Hk, _Hp, _DtmClass, _, _Maxml ).
distribution_has_maximal_kategory( [Hk-Hp|T], MxK, MxP, TrueK, TKPrb, Maxml ) :-
	( TrueK == Hk ->
		TKPrb = Hp,
		( (Hp >= MxP,kv_no_larger_v(T,Hp)) -> 
				Maxml = true
				;
				Maxml = false
		),
		Rem = []
		;
		Rem = T,
		( Hp =< MxP -> 
			NxMxK = MxK,
			NxMxP = MxP
			;
			NxMxK = Hk,
			NxMxP = MxP 
		)
	),	
	distribution_has_maximal_kategory( Rem, NxMxK, NxMxP, TrueK, TKPrb, Maxml ).
	
kv_no_larger_v( [], _Max ).
kv_no_larger_v( [_-Kv|T], Max ) :-
	Kv =< Max,
	kv_no_larger_v( T, Max ).

	/* 
	%%% HERE
	( memberchk( DtmClass-ClassPrb, Distro ) -> 
		true
		;
		write( user_error, unrecognised_class(ElClass) ), 
		nl( user_error )
	),
	distribution_to_correct( Distro, ClassPrb, ElClass, Corr ).

distribution_to_correct( [], _ElPrb, _ElClass, Corr ) :-
	( var(Corr) -> Corr = true; true ).
distribution_to_correct( [HCl-HPrb|T], PrbFound, TrueClass, Corr ) :-
	( HPrb < TruePrb -> 
		Rm = T
		;
		( (HPrb =:= PrbFound, HCl == TrueClass) -> 
				Rm = T
				;
				Corr = false,
				Rm = []
		)
	),
	distribution_to_correct( Rm, TruePrb, TrueClass, Corr ).
	*/

% 
cart_to_classifier( Cart, Ids, Adj, Classifier ) :-
	( (Cart = cart(_,_,_,_);Cart=leaf(_,_)) -> 
		% only for backward compatibility
		cart_to_classifier_full( Cart, Adj, Classifier )
		;
          find_first_existing_feature_vector( 1, FeatVect ),
          ( looks_like_fprs_vector(FeatVect,Ids) ->
		     % write( cart_to_classifier_red_fprs( Cart, Ids, Adj, Classifier ) ), nl,
		     cart_to_classifier_red_fprs( Cart, Ids, Adj, Classifier )
               % write( classifier(Classifier) ), nl
               ;
		     cart_to_classifier_red( Cart, Ids, Adj, Classifier )
          )
	).

cart_to_classifier_full( cart(F,Fvl,L,R), Adj, cart(F,Fvl,NwL,NwR) ) :-
	cart_to_classifier_full( L, Adj, NwL ),
	cart_to_classifier_full( R, Adj, NwR ).
cart_to_classifier_full( leaf(_D,Ids), Adj, leaf(Distro) ) :-
	leaf_distribution( Ids, Adj, Distro ).

cart_to_classifier_red( cr(F,Fvl,L,R), Ids, Adj, cart(F,Fvl,NwL,NwR) ) :-
	% from ids_dichotomy in ids_split.slp :
	bb_get( F, FIndex ), 
	memberchk( Fvl-PointIds, FIndex ),
	ord_intersection( PointIds, Ids, LIds, RIds ),
	cart_to_classifier_red( L, LIds, Adj, NwL ),
	cart_to_classifier_red( R, RIds, Adj, NwR ).
cart_to_classifier_red( l, Ids, Adj, leaf(Distro) ) :-
	% P(Y="yes"|x,T) = (n_{yes}+1)/(n_{yes}+n_{no}+2)
	leaf_distribution( Ids, Adj, Distro ).
	%%% HERE leaf_distribution_red( Ids, Adj, Distro ).
	% ids_to_dominant_class( Ids, [], DClass ).

cart_to_classifier_red_fprs( cr(F,Fvl,L,R), Ids, Adj, cart(F,Fvl,NwL,NwR) ) :-
	% from ids_dichotomy in ids_split.slp :
	bb_get( F, FIndex ), 
     cart_fprs_select_and_separate_values( Ids, FIndex, Fvl, LIds, RIds ),
	% memberchk( Fvl-PointIds, FIndex ),
	% ord_intersection( PointIds, Ids, LIds, RIds ),
	cart_to_classifier_red_fprs( L, LIds, Adj, NwL ),
	cart_to_classifier_red_fprs( R, RIds, Adj, NwR ).
cart_to_classifier_red_fprs( l, Ids, Adj, leaf(Distro) ) :-
	% P(Y="yes"|x,T) = (n_{yes}+1)/(n_{yes}+n_{no}+2)
	leaf_distribution( Ids, Adj, Distro ).
	%%% HERE leaf_distribution_red( Ids, Adj, Distro ).
	% ids_to_dominant_class( Ids, [], DClass ).

cart_fprs_select_and_separate_values( [], _KVs, _Val, [], [] ).
cart_fprs_select_and_separate_values( [Id|TId], [HId-HVal|TPrs], Pvt, L, R ) :-
     !,
     ( Id == HId ->
          RId = TId,
          ( HVal > Pvt ->
               R = [Id|RT],
               L = LT
               ;
               R = RT,
               L = [Id|LT]
          )
          ;
          RId = [Id|TId],
          LT = L, RT = R
     ),
     cart_fprs_select_and_separate_values( RId, TPrs, Pvt, LT, RT ).

leaf_distribution( Ids, Adj, Distro ) :- 
	bb_get( all_categories, Kats ),
	( Adj == none -> Marg is 0; Marg is 1 ),
	kv_compose_fixed_v( Kats, Marg, KatsCnts ),
	have_categories( Ids, IdsKats ),
	kvs_incr_v_on_keys( IdsKats, KatsCnts, CntPrs ),
	kv_sum_vs( CntPrs, NumOfIds ),
	/* The else (Adj==bayes) part is wrong. 
	   We dont need the NumOfIds + NumOfKats part since
	   we already added one to each KatCnt. When summing CntPrs
	   we, thus, get the correct divisor.
	( Adj == none -> Div is NumOfIds;
		length( Kats, NumOfKats ),
		Div is NumOfIds + NumOfKats ),
	kv_v_term_div( CntPrs, Div, Distro ).
		*/
	kv_v_term_div( CntPrs, NumOfIds, Distro ).

have_categories( [], [] ).
have_categories( [H|T], [Hkat|TKats] ) :-
	once( has_category( H, Hkat ) ),
	have_categories( T, TKats ).

kv_v_term_div( [], _By, [] ).
kv_v_term_div( [K-V|T], By, [K-(V/By)|TDiv] ) :-
	kv_v_term_div( T, By, TDiv ).

/*
ids_to_dominant_class( [], Pairs, DClass ) :-
	kv_decompose_on_max_v( Pairs, MaxVks, _MaxVvs ),
	( MaxVks = [DClass] -> 
		true
		;
		length( MaxVks, Length ),
		select_n_uniformly( 1, MaxVks, Length, [DClass], _ )
	).
ids_to_dominant_class( [H|T], Acc, DClass ) :-
	has_category( H, HClass ),
	( select(HClass-Tms,Acc,RmAcc) -> 
		Inc is Tms + 1,
		NxAcc = [HClass-Inc|RmAcc]
		;
		NxAcc = [HClass-1|Acc]
	),
	ids_to_dominant_class( T, NxAcc, DClass ).
	*/
find_first_existing_feature_vector( 100, _FeatVect ) :- !, fail.
find_first_existing_feature_vector( I, FeatVect ) :-
     bb_get( I, FeatVect ),
     !.
find_first_existing_feature_vector( I, FeatVect ) :-
     NxI is I + 1,
     find_first_existing_feature_vector( NxI, FeatVect ).

looks_like_fprs_vector( [], [] ).
looks_like_fprs_vector( [Id-_Val|T], [Id|TIds] ) :-
     looks_like_fprs_vector( T, TIds ).
