:- ensure_loaded( library(lists) ).		% append/3, memberchk/2, is_list/1.
:- ensure_loaded( library(ordsets) ).		% ord_add_element/3.
:- ensure_loaded( library(system) ).		% environ/2, datime/1,
									% delete_file/1, system/1.
:- ensure_loaded( '../../src/init_lib' ).

:- ensure_loaded( library(random) ).		% setrand/1.
                                    		% leave this AFTER ini_lib (SWI).
:- elm( 'auxil/n_samples' ).
:- elm( 'auxil/seeds' ).			% seed/2.
:- pl( swi(_), true, 
               ensure_loaded(library(flatten)) ). % /2.
:- ensure_loaded( library(flatoms) ).			% /2.
:- ensure_loaded( library(number_abbreviation) ).	% /2.
% :- ensure_loaded( library(slp_in_options) ). 	% /2.
:- ensure_loaded( library(kv_idks_max_v) ). 		% /3.
:- ensure_loaded( library(basename) ). 			% /2.
:- ensure_loaded( library(werr) ). 			% /1.
:- ensure_loaded( library(break_nth) ).			% /4.
:- ensure_loaded( library(list_frequency) ).		% /2.
:- requires( to_list/2 ).

:- ensure_loaded( loss ). 			% bn_eg/2.
:- ensure_loaded( 'auxil/the_true_bn_str_of' ).

compare :-
	compare( [] ).

% Asia : 1-[],2-[],3-[1],4-[2],5-[2],6-[3,4],7-[5,6],8-[6]
% Opts has data(DFile), mcmcms(MCMSLoc), res(ResDir), sleep(Secs).
compare( Opts ) :-
	( (memberchk( help, Opts ) ; Opts == help ) -> 
		compare_help
		; 
		memberchk( help(Help), Opts ),
		( Help == cdata ->
			compare_help_cdata
			;
			werr( [['Unrecognised help/1 argument. Please use cdata.']] )
		)
	),
	!.
compare( PrvOpts ) :-
     to_list( PrvOpts, Opts ),
	compare_defaults( Opts, Defs ),
	append( Opts, Defs, AllOpts ),
	memberchk( res(Res), AllOpts ),
	memberchk( sleep(SlSecs), AllOpts ),
	memberchk( mcmcms(MCMSLoc), AllOpts ),
	memberchk( data(DFile), AllOpts ),
	memberchk( seed(Seed), AllOpts ),
	seed( Seed, rand(S1,S2,S3) ),
	setrand( rand(S1,S2,S3) ),
	memberchk( loss_skip(Skip), AllOpts ),
	ensure_loaded( DFile ),
	bb_put( sleep_interval, SlSecs ),
	bb_put( mcmcms_location, MCMSLoc ),
	make_directory( Res ),
	atom_concat( Res, '/comparison_data.pl', ResDFile ),
	( file_exists(DFile) -> 
		cp( DFile, ResDFile )
		; 
		atom_concat( DFile, '.pl', DFilePl ),
		cp( DFilePl, ResDFile )
	),
	findall( cd(Ls,Tr,DtSz,MS,BS,RL,Prr,Lhd,Sd,ST,XFs,SGl), 
		comparison_data(Ls,Tr,DtSz,MS,BS,RL,Prr,Lhd,Sd,ST,XFs,SGl), CData ),
	findall( XFs, 
		member( cd(_Ls,_Tr,_DtSz,_MS,_BS,_RL,_Prr,_Lhd,_Sd,_TS,XFs,_SGl), CData ),
			XtraFTrms ),
	flatten( XtraFTrms, FlatFTrms ),
	files_to_copy( FlatFTrms, ToCopy ),
	do_the_data( Res, CData, ToCopy ),
     write( done_the_data ), nl,
	working_directory( Old, Res ),
	open( lscript, write, S ),
	write( S, '#!/bin/sh' ), nl( S ), nl( S ),
	atom_concat( Old, '/', OlD ),
	memberchk( eng(Engine), AllOpts ),
	to_script( CData, S, OlD, Res, Skip, Engine ),
	close( S ),
	system( 'chmod u+x lscript' ),
	working_directory( _New, Old ),
	write( 'Script, lscript, created in directory ' ),
	write( Res ), nl, !.
compare( _ ) :-
	werr( [['Error encountered while runing compare/1.']] ).

do_the_data( Res, CData, ToCopy ) :-
	atom_concat( Res, '/true', TrueDir ), 
	atom_concat( Res, '/data', DataDir ), 
	atom_concat( Res, '/slps', SlpsDir ), 
	atom_concat( Res, '/models', ModDir ), 
	atom_concat( Res, '/losses', LosDir ), 
	atom_concat( Res, '/auxil',  AuxDir ), 
	make_directory( TrueDir ),
	make_directory( DataDir ),
	make_directory( SlpsDir ),
	make_directory( ModDir ),
	make_directory( LosDir ),
	make_directory( AuxDir ),
	cp( 'loss.pl', Res ),
	cp( 'expected_loss.pl', Res ),
	cp( 'auxil/post_loss.pl', Res ),
	atom_concat( Res, '/', ResSl ),
	cp_all_to( ToCopy, ResSl ),
	findall( (Tr,MS)-DtSz,
		member( cd(_Ls,Tr,DtSz,MS,_BS,_RL,_Prr,_Lhd,_Sd,_TS,_XFs,_SGl), CData ),
			Pairs ),
	kv_idks_max_v( Pairs, [], MxPairs ),
	sample_pairs_recording( MxPairs, Res, Pairs ).

sample_pairs_recording( [], _Res, _Ps ).
sample_pairs_recording( [(True,MS)-Max|T], Res, Pairs ) :-
	bb_put( msd, MS ),
	sample_n_from_bn_in( True, Max, TFile, Sampled ),
	findall( Point, member((True,MS)-Point,Pairs), Points ),
	atom_concat( Res, '/true/', ResTrue ),
	cp( TFile, ResTrue ),
	atom_concat( Res, '/data', ResData ),
	record( Points, True, ResData, Sampled ),
	sample_pairs_recording( T, Res, Pairs ).

to_script( [], _S, _OlD, _Res, _Skip, _Engine ).
to_script( [H|T], S, OlD, Res, Skip, Engine ) :-
	H = cd(PvLFs,Tr,DtSz,MS,BS,RL,Prr,Lhd,Sd,ST,XFs,SGl),
	( is_list(PvLFs) -> LFs = PvLFs ; LFs = [PvLFs] ),
	( results_file_name( Tr, DtSz, MS, BS, RL, Prr, Lhd, Sd, RFile, MkDir ) ->
		true
		;
		Mess = [['Unable to construct results filename.'],
	             ['Failing...']
	           ],
		werr( Mess ), 
		fail
	),
	( MkDir == none -> true ; write( S, 'mkdir ' ), 
		write( S, MkDir ), nl( S ) ),
	SGl = _/[Nodes|_],
	nodes_to_data_format( Nodes, DFrm ),
	the_true_bn_str_of( Tr, TBn ),
	bn_eg( TBn, TEG ),
	write( S, 'umask 022' ), nl( S ),
     pl_engine_to_start_csh_script( Engine, CshScrStart ),
	write( S, CshScrStart ),
	write( S, '  << end_of_this_one' ),% write( S, RFile ), 
	nl( S ),
	write( S, '[user].' ), nl( S ),
	write( S, 'true_bn(' ), 
	writeq( S, TBn ),
	write( S, ').' ), nl( S ),
	write( S, 'true_eg(' ), 
	writeq( S, TEG ),
	write( S, ').' ), nl( S ),
	( (ST == [];ST == none) ->
		true
		;
		TSCall =.. [ST,S],
		call( TSCall )
	),
	write( S, DFrm ), write( S, '.' ), nl( S ),
	write( S, 'end_of_file.' ), nl( S ),
	bb_get( mcmcms_location, MCMCMS ),
	load_extra_files( XFs, S ),
	write( S, compile( loss ) ), write( S, '.' ), nl( S ),
	write( S, compile( expected_loss ) ), write( S, '.' ), nl( S ),
	pl( swi(_), write(S,'consult(\''), write(S,'compile( \'' ) ),
     write( S, MCMCMS ),
	write( S, '\' ).' ), nl( S ),
	( (MS==rm;MS==fm) ->
		write( S, 'bb_put( msd, ' ), 
		write( S, MS ), write( S, ' ).' ), nl( S ) 
	  	;
		Mess = [['Unknown model space definition \'',MS,'\'.'],
	             ['Using rm.']
	           ],
		werr( Mess ),
		write( S, 'bb_put( msd, rm ).' ), nl( S )
	),
	% write( S, 'bb_put( dbg_these, [24,28,29,55] ).' ), nl( S ),
	number_abbreviation( DtSz, SzAbb ),
	flatoms( ['data/',Tr,'_',SzAbb,'.pl'], DtF ),
	( vrs_file_exists( OlD, Tr, VrsF ) ->
		atom_concat( Res, '/true/', ResTrue ),
		cp( VrsF, ResTrue ), 
		atom_concat( 'true//', Tr, TrDirTr ),
		atom_concat( TrDirTr, '_vrs', TDTVrc ),
		write( S, 'compile( \'' ), write( S, TDTVrc),
		write( S, '\' ).' ), nl( S )
		;
		true
	),
	write( S, 'write( \'compiling the data ...\' ),' ), nl( S ),
	write( S, 'nl,' ), nl( S ),
	write( S, 'compile( \'' ), write( S, DtF ),
	write( S, '\' ).' ), nl( S ),
	write( S, 'write( done ), nl,' ), nl( S ),
	atom_concat( OlD, Lhd, FullLhd ),
	absolute_file_name( FullLhd, AbsLhd ),
	writeq( S, compile(AbsLhd) ), write( S, '.' ), nl( S ),
	basename( Prr, PrrBase ), 
	atom_concat( 'slps/', PrrBase, SlpsPrr ),
	atom_concat( OlD, Prr, FullPrr ),
	cp( FullPrr, SlpsPrr ),
	write( S, 'dbg_these( [28] ).' ), nl( S ),
	% write( S, 'dbg_these( [24,28] ).' ), nl( S ),
	% write( S, 'dbg_these( [24,28,29] ).' ), nl( S ),
	write( S, 'mcmcms(' ), 
	writeq( S, RFile ), write( S, ', ' ),
	write( S, '\'' ), write( S, RFile ), write( S, '.stats\', ' ),
	BS =.. [BSName|BSArgs],
	( BSArgs=[PrbArg|_] -> true ; PrbArg = inf ),
	write( S, BSName ), write( S, ', ' ),
	writeq( S, SlpsPrr ), write( S, ', ' ),
	writeq( S, PrbArg ), write( S, ', ' ),
	writeq( S, RL ), write( S, ', ' ),
	write( S, null ), write( S, ', ' ),
	write( S, [] ), write( S, ', ' ),
	seed( Sd, rand(S1,S2,S3) ),
	write( S, S1), write( S, ', ' ),
	write( S, S2), write( S, ', ' ),
	write( S, S3), write( S, ', ' ),
	writeq( S, SGl ),
	write( S, ' ).' ), nl( S ),
	% losses_ids_to_script( LFs, S ),
	writeq( S, expected_loss(LFs,Skip,losses,RFile) ), write( S, '.' ), nl( S ),
     % pl( swi(_), true, (write( S, 'halt.' ), nl( S )) ),
     % 2007/02/16, current YAP doesnt seem to need the above, 
     %             I dont know if it is required in SICStus
	write( S, 'end_of_this_one' ), nl( S ), nl( S ), 
	bb_get( sleep_interval, SI ),
	write( S, 'gzip ' ), write( S, RFile ), nl( S ), nl( S ),
	( T == [] -> 
		write( S, 'echo Done. ; chmod -x ./lscript' ), nl( S )
		;
		write( S, 'sleep ' ), write( S, SI ), nl( S )
	),
	!,
	to_script( T, S, OlD, Res, Skip, Engine ).
to_script( [H|T], S, OlD, Res, Skip, Engine ) :-
	werr( [['Error while processing comparison_data/n clause: ',H],['Script file probably corrupted.'],['However, processing is continued.']] ),
	to_script( T, S, OlD, Res, Skip, Engine ).

% losses_ids_to_script( [], _S ).
% losses_ids_to_script( [H|T], S ) :-
	% loss_id( H, Hid ),
	% writeq( S, assert(loss_id(H,Hid)) ), write( S, '.' ), nl( S ),
	% losses_ids_to_script( T, S ).

results_file_name( Tr, DtSz, MS, BS, RL, Prr, Lhd, Sd, RFile, MkDir ) :-
	% loss_id( Ls, LsId ),
	model_id( Tr, TrId ),
	basename( Prr, BsPrr ),
	prior_id( BsPrr, PrId ),
	lhood_id( Lhd, LhdId ),
	number_abbreviation( DtSz, Sz ),
	bs_to_id( BS, BSAtm ),
	number_abbreviation( RL, Length ),
	number_codes( Sd, SdCs ), 
	atom_codes( SId, SdCs ),
	flatoms( ['models/cm_t',TrId,'_d',Sz,'_m',MS,'_b',BSAtm,
	          '_r',Length,'_p',PrId,'_h',LhdId,'_s',SId], RFile ),
	MkDir = none.

compare_defaults( Opts, [
		res(Res),
		sleep(10),
		mcmcms(MSLoc),
		loss_skip(500),
		data('cdata/comparison_data'),
		eng(Eng),
		seed(1)
		] ) :-
	pl( Pl ), functor( Pl, Eng, _Arity ),
	datime( datime(Y,M,D,_Hr,_Min,_Sec) ),
	numbers_to_two_codes_min( [Y,M,D], [YCs,MCs,DCs] ),
	( environ(mcr,Pfx) -> 
		atom_codes( Pfx, PfxCs )
		;
		PfxCs = "/tmp"
	),
	( memberchk(dir_pfx(DirPfx),Opts) ->
		atom_codes( DirPfx, DirPfxCs )
		;
		DirPfxCs = "amai"
	),
	flatten( [PfxCs,"/",DirPfxCs,"_",YCs,MCs,DCs], ResCs ),
	atom_codes( Res, ResCs ),
	working_directory( Here, Here ),
	atom_concat( Here, '/../mcmcms.pl', RelMSLoc ),
	absolute_file_name( RelMSLoc, MSLoc ).

numbers_to_two_codes_min( [], [] ).
numbers_to_two_codes_min( [H|T], [HCs|TCs] ) :-
	number_codes( H, PrvHCs ),
	( PrvHCs = [Single] -> 
		HCs = [0'0|Single]
		;
		HCs = PrvHCs 
	),
	numbers_to_two_codes_min( T, TCs ).

vrs_file_exists( Old, Tr, VrsFl ) :-
	atom_concat( Tr, '_vrs.pl', PsxPl ),
	atom_concat( '/true/', PsxPl, PsxTr ),
	atom_concat( Old, PsxTr, VrsFl ),
	file_exists( VrsFl ).


%%% data auxils 
record( [], _True, _ResDir, _ ).
record( [H|T], True, ResDir, Sampled ) :-
	number_abbreviation( H, HAbb ),
	flatoms( [ResDir,'/',True,'_',HAbb,'.pl'], TrueRec ),
	break_nth( H, Sampled, FirstNSampled, _ ),
	list_frequency( FirstNSampled, FrqOfFirstN ),
	open( TrueRec, write, Out ),
	record_sampled_frq( FrqOfFirstN, Out ),
	close( Out ),
	record( T, True, ResDir, Sampled ).

% Change 1.
record_sampled_frq( [], _Out ).
record_sampled_frq( [DataList-Tms|T], Out ) :-
	Data =.. [data,DataList,Tms],
	write_canonical( Out, Data ), write( Out, '.' ), nl( Out ),
	record_sampled_frq( T, Out ).

/* remove:
record_first_n_samples( 0, _Sampled, _Out ) :- !.
record_first_n_samples( I, [H|T], Out ) :-
	I > 0,
	Data =.. [data|H],
	write_canonical( Out, Data ), write( Out, '.' ), nl( Out ),
	NxI is I - 1,
	record_first_n_samples( NxI, T, Out ).
	*/

sample_n_from_bn_in( True, N, TFile, Sampled ) :-
	atom_concat( 'parity', _, True ),
	!,
	atom_concat( True, '.slp', TrueSlp ),
	atom_concat( 'true/', TrueSlp, TFile ),
	( N > 2500 -> 
		werr( [['Cannot do',N,' samples for parity sets because there are only 2500 presampled points.'],['Aborting...']] ), abort
		;
		atom_concat( True, '_2500_data.pl', ParityPl ),
		atom_concat( 'true/', ParityPl, ParityD ),
		parity_data_to_sampled( ParityD, N, Sampled )
	).

sample_n_from_bn_in( True, N, TFile, Sampled ) :-
	% slp_in_options( True, TFile ),
	atom_concat( True, '.slp', TrueSlp ),
	atom_concat( 'true/', TrueSlp, TFile ),
	n_samples( [slp(TFile),samples(N),call(sample(V)-V),res(nat),
	            ad_to_slp([msd(rm),load(cn),rm(true)])], _Tmp, Sampled ),
                 write( done_samples(TFile) ), nl.

% other auxils
cp( From, To ) :-
	flatoms( ['cp ',From,' ',To], Cp ),
	system( Cp ).

nodes_to_data_format( Nodes, DFrm ) :-
	anonymise_list( Nodes, Anonymoi ),
	Data =.. [data|Anonymoi],
	DFrm =.. [data_format,Data].

pl_engine_to_start_csh_script( sicstus, 'sicstus -f ') :- !.
pl_engine_to_start_csh_script( swi, 'pl -f f ') :- !.
pl_engine_to_start_csh_script( yap, 'yap -f ').

anonymise_list( [], [] ).
anonymise_list( [_H|T], ['_'|R] ) :-
	anonymise_list( T, R ).

bs_to_id( BS, BSAtm ) :-
	BS =.. [Name|Rest],
	( Rest = [] -> 
		BSAtm = Name
		;
		Rest = [N|_],
		number_codes( N, NCs ),
		( NCs = [0'0,0'.|DecCs] ->
			atom_codes( BSNumAtm, DecCs )
			;
			atom_codes( BSNumAtm, NCs )
		),
		atom_concat( Name, BSNumAtm, BSAtm )
	).

% help message
compare_help :-
	write_atms([
'compare/1 is a Prolog program generating a new directory and script file for',
'    running a number of mcmcms experiments with integral computation',
'    of expected loss for each experiment. The generated script is in',
'    unix /bin/sh language. The main reason is that we want the ',
'    execution of each experiment to be done as cleanly as possible,',
'    thus we run a separate prolog engine for each experiment.',
'    The program is quite generic but it still contains some hardwired',
'    references to MCMCMS runs over BNs.',
'',
'    The following options are recognised :',
'             ',
'    data(DFile),     the data file to be loaded. Provides comparison_data/11',
'                     loss_id/2, model_id/2, prior_id/2 and lhood_id/2.',
'                     One experiment, for each derivation of comparison_data/n',
'                     will be created. Default is',
'                     cdata/comparison_data.pl.  (This file doesnt exist',
'                     in current distribution.)',
'    help,            print this help message',
'    help(cdata),     print help message for comparison_data/n',
'    mcmcms(MSLoc),   the full location for \'mcmcms.pl\'. Default is ',
'                     \'../mcmcms.pl\'.',
'    res(ResDir),     directory where all the set-up files for the ',
'                     experiments are placed. Default is ',
'                     <Pfx>/<DirPfx>_<Date>, where Date is in YYMMDD',
'                     format and Pfx is either equal to $MCMCMS_REPO',
'                     if this variable is defined, or to',
'                     \'/local/d0p6/$USER\', otherwise. Default value of',
'                     DirPfx is \'cmpr\' which can be changed by dir_pfx/1.',
'                     If directory ResDir exists, an error is printed and',
'                     the execution is aborted.',
'    dir_pfx(DirPfx), directory prefix; set to \'cmpr\' by default.',
'    sleep(Secs),     number of seconds the script process should sleep',
'                     for, betweeen each experiment. Default is Secs=10.',
'    loss_skip(Skp)   how many of the initial models in chain should be',
'                     excluded when calculating expected losses.',
'',
'    To test use  ?- compare([data(\'cdata/simple_test\')]).',
'    then         % cd <created_dir>',
'    and          % ./lscript',
'',
'    If you have Upsh installed on your system the first line of the',
'    example above can be changed to:',
'                     % upsh f compare data=atm=cdata/simple_test',
'',
'    The following files and directories are created in ResDir:',
'            ',
'         expected_loss.pl  provides computation of expected loss.',
'         loss.pl           the loss functions.',
'         lscript     the /bin/sh script to run. This executable, but',
'                           after it is run once it self changes its',
'                           permisssions to non-executable.',
'',
'         data/             contains all data sampled (sampling takes place',
'                           at creation of script so this directory will',
'                           be populated at creation).',
'         losses/           for each loss function a .lID and a .eID will be',
'                           created by the script. The former holds all losses',
'                           for visited models in experiment, while the',
'                           latter records the overall expected loss of the',
'                           run according to the specified loss function.',
'                           For each loss function in comparison_data/n',
'                           a unique ID should be used.',
'         models/           for each experiment record visited and proposed',
'                           models (c/1 and b/1 terms respectively in .gz',
'                           format) and some statistics about the run',
'                           in .stats file.',
'         slps/             all the prior defining .slp files.',
'         true/             all the true models. Each should define a ',
'                           sample/1 clause which will be used for sampling.',
'                           This will be called with a variable as its ',
'                           only argument. This currently is hardwired to',
'                           comparing BNs, and it instantiates to a list',
'                           of values for the variables in the network.',
'                           Furthermore, the true_bn/1 is constructed',
'                           automatically from this file, so it should be',
'                           in the format specified in compare/true/test.pl.']
					  ).

compare_help_cdata :- 
	write_atms([ 
'comparison_data( LF, True, S, ModSpc, BStg, Run, Prr, Lhd, SId, TS, G ) :-',
'',
'   LF:     List of loss function names. Each element should be the name of a',
'           binary predicate defined in loss.pl.',
'',
'   True:   Slp file containing the true model (for now the true BN). Format',
'           should be as shown in true/test.slp. This SLP is used for sampling',
'           and constructing the true_bn/1 model.',
'',
'   S:      Number of samples for the experiment.',
'   ',
'   ModSpc: Model space backtracking, either rm for Real Models only, or fm ',
'           including Failure, imaginary, Models too. Note that this is used',
'           both for sampling and for the MCMC.',
'   ',
'   BStg:   Backtracking strategy. As in run_data/10 second argument. See ',
'           doc/user_guide.ps',
'',
'   Run:    Number of visited models in the chain, i.e. iterations of the ',
'           Metropolis-Hastings algorithm.',
'',
'   Prr:    SLP encoding the prior. ',
'',
'   Lhd:    Likelihood file as expected my mcmcms.pl.',
'',
'   SId:    Seed Id, an integer such that 0< SId < 1001. Corresponds to a triplet',
'           of numbers setting the seed for the random number generator. See file',
'           mcmcms/auxil/seeds.pl.',
'',
'   TS:     to script. Predicate name of an unary call which takes the out stream',
'           of the script as its only argument. Use none or [] for no call.',
'',
'   Goal:   The goal structure to call for the MCMC iterations. Similar to the',
'           structure in the sixth argument of run_data/10. Typically, here it ',
'           will be of the form bn/[[1,2,...n]] where n is the number of variables',
'           in the BN. ',
'',
'When filenames are expected in the above, i.e. True, Prr and Lhd the relative',
'location from the current directory should be given.'] ).

write_atms( [] ).
write_atms( [H|T] ) :-
	write( H ), nl,
	write_atms( T ).

cp_all_to( [], _ResSl ).
cp_all_to( [cp(F,Sub)|T], ResSl ) :-
	atom_concat( ResSl, Sub, DestDir ),
	atom_concat( 'cp ', F, CpF ),
	atom_concat( ' ', DestDir, SpcDestDir ),
	atom_concat( CpF, SpcDestDir, Copy ),
	( system( Copy ) -> 
		true
		;
		write( user_error, failed_copy(Copy) ), nl( user_error )
	),
	cp_all_to( T, ResSl ).

parity_data_to_sampled( ParityD, N, Sampled ) :-
	open( ParityD, read, In ),
	parity_data_stream_to_sampled( N, In, Sampled ),
	close( In ).

parity_data_stream_to_sampled( 0, _In, Tail ) :- !, Tail = [].
parity_data_stream_to_sampled( N, In, [H|T] ) :-
	read( In, Data ),
	Data =.. [data|H],
	NxN is N - 1,
	parity_data_stream_to_sampled( NxN, In, T ).
	
files_to_copy( [], [] ).
files_to_copy( [H|T], ToCopy ) :-
	( to_copy( H, F, Dir ) ->
		ToCopy = [cp(F,Dir)|TTCpy]
		;
		ToCopy = TTCpy
	),
	files_to_copy( T, TTCpy ).

to_copy( cp(F,Dir), F, Dir ).
to_copy( cp(F), F, 'auxil/' ).
to_copy( cp_ld(F,Dir), F, Dir ).
to_copy( cp_ld(F), F, 'auxil/' ).

to_load( ld(F,Dir), F, Dir ).
to_load( ld(F), F, 'auxil/' ).
to_load( cp_ld(F,Dir), F, Dir ).
to_load( cp_ld(F), F, 'auxil/' ).

load_extra_files( [], _S ).
load_extra_files( [H|T], S ) :-
	( to_load(H,F,Dir) -> 
		basename( F, Base ),
		atom_concat( Dir, Base, LdFile ),
		writeq( S, compile( LdFile ) ),
		write( S, '.' ), nl( S )
		;
		true
	),
	load_extra_files( T, S ).
