% This is very similar to stepped. In addition a list of known root causes,
% KnRtCaus, is expected. This list, contains all nodes which are parentless
% in the true BN.
% Note that the probability distribution indices (PopIndc) should
% no longer contain the first index since this corresponds to the 
% known causes.

:- ensure_loaded( library(ordsets) ).		% list_to_ord_set/2.
:- ensure_loaded( library(lists  ) ).		% memberchk/2.

bn( Nodes, KnRtCaus, PopIndc, Bn ) :-
	length( Nodes, NofAllNds ),
	length( KnRtCaus, NofKnRt ),
	NofNds is NofAllNds - NofKnRt,   % double check this...
	bn_1( Nodes, Nodes, KnRtCaus, PopIndc, NofNds, Bn, TBn ),
	TBn = [].

bn_1( [], _AllNds, _KnRtCaus, _Indc, _Dnm, TBn, TBn ) :- !.
bn_1( Nds, AllNds, KnRtCaus, Indc, Dnm, Bn, TBn ) :-
	length( Nds, Lgth ),
	[Lgth] ## psplit( Nds, Nd, Left, Right ),
	bn_1( Left, AllNds, KnRtCaus, Indc, Dnm, Bn, TLeft ),
	nth_split( 1, Nd, AllNds, PotPa ),
	choose_pa_on_known( Nd, KnRtCaus, Indc, PotPa, Dnm, UnoPa ),
	list_to_ord_set( UnoPa, Pa ),
	TLeft = [Nd-Pa|RInp],
	bn_1(  Right, AllNds, KnRtCaus, Indc, Dnm, RInp, TBn ).

% bn_1(  [], _KnRtCaus, _EmLst, _AllNds, _Zero, [] ).
% bn_1(  [Nd|Nds], KnRtCaus, Indc, PotPa, Dnm, [Nd-Pa|TBn] ) :-
	% 
	% choose_pa_on_known( Nd, KnRtCaus, Indc, PotPa, Dnm, UnoPa ),
	% list_to_ord_set( UnoPa, Pa ),
	% bn_1(  Nds, KnRtCaus, Indc, [Nd|PotPa], Dnm, TBn ).

choose_pa_on_known( Nd, KnRtCaus, _Indc, _PotPa, _Dnm, Pa ) :-
	memberchk( Nd, KnRtCaus ),
	!,
	Pa = [].
choose_pa_on_known( _Nd, _KnRtCaus, Indc, PotPa, Dnm, [FstPa|TPa] ) :-
	pselect( PotPa, FstPa, RemPotPa ),
	choose_pa_if( RemPotPa, Indc, Dnm, TPaUno ),
	list_to_ord_set( TPaUno, TPa ).

choose_pa_if( _PotPa, [], _Dnm, [] ) :-
	!.
choose_pa_if( [], _Ind, _Dnm, [] ) :-
	!.
choose_pa_if( PotPa, [Nom|TInd], Dnm, Pa ) :-
	[Nom,Dnm] ## choose_pa( PotPa, TInd, Pa ).
	% rselect( PotPa, OnePa, RemPotPa ),
	% random( Rnd ),
	% on_random_choose_pa_if( Rnd, Nom, Dnm, RemPotPa, TInd, TPa ).

% on_random_choose_pa_if( Rnd, Nom, Dnm, _RemPotPa, _TInd, TPa ) :-
	% Rnd < Nom/Dnm,
	% !,
	% TPa = [].
% on_random_choose_pa_if( _Rnd, Nom, Dnm, RemPotPa, TInd, TPa ) :-
	% NxDnm is Dnm - Nom,
	% [Nom,Dnm] :: choose_pa_if( RemPotPa, TInd, NxDnm, TPa ).
% 
	% choose_pa( PotPa, TInd, Pa ).
 
:- s_no_bpoint( choose_pa/3 ).

Nom/Dnm :: [Nom,Dnm] :: choose_pa( _PotPa, _Indc, [] ).
1 - (Nom/Dnm) :: [Nom,Dnm] :: choose_pa( PotPa, Indc, [OnePa|Pa] ) :-
	pselect( PotPa, OnePa, RemPotPa ),
	NxDnm is Dnm - Nom,
	choose_pa_if( RemPotPa, Indc, NxDnm, Pa ).

:- s_tail_recursive( psplit/4 ).

1/X :: X :: psplit( [H|T], H, [], T ).
(1 - 1/X) :: X :: psplit( [H|T], El, [H|L], R ) :-
				[(X - 1)]:: psplit( T, El, L, R ).

:- s_no_bpoint( pselect/3 ).
:- pvars( pselect(L,_E,_R), [T-length(L,T)] ). 

1/X :: X :: pselect( [H|T], H, T ).
(1 - 1/X) :: X :: pselect( [H|T], El, [H|R] ) :-
                                [(X - 1)]:: pselect( T, El, R ).

nth_split( Nth, Nth, _Tail, Left ) :- !, Left = [].
nth_split( Curr, Nth, [H|T], [H|TLeft] ) :-
	NxCurr is Curr + 1,
	nth_split( NxCurr, Nth, T, TLeft ).
