:- ensure_loaded( library(lists) ).		% append/3, memberchk/2.
:- ensure_loaded( '../../../src/init_lib' ).

:- ensure_loaded( library(requires) ).
:- requires( directory_contents/4 ).
:- requires( write_list_of_lists_to_spaced_line_per_skim/2 ).
:- requires( file_stems_of_files_with_this_ext/2 ).
:- requires( longest_prefix/3 ).
:- requires( sys_info/1 ).
:- requires( flatten/2 ).
:- requires( to_number/2 ).
:- requires( gnucomfile/0 ).		% dodgy use of requires/1

mk_versus :-
	mk_versus( [] ).

mk_versus( Opts ) :-
	xy_opts( Opts, Xmax, Ymax, Xpad, Ypad ),
	directory_contents( '.', Files, _Dirs, _Oth ),
	Pct = '.predicted',
	file_stems_of_files_with_this_ext( Files, Pct, PctStems ),
	files_n_first_feature_prediction_pairs( PctStems, Pct, Pcts ),
	mk_versus_list( Pcts, Xmax/Ymax ),
	Pstv = '.predicted_pos',
	file_stems_of_files_with_this_ext( Files, Pstv, PNStems ),
	files_n_first_feature_prediction_pairs( PNStems, Pstv, Pstvs ),
	Ngtv = '.predicted_neg',
	files_n_first_feature_prediction_pairs( PNStems, Ngtv, Ngtvs ),
	mk_pn_versus_lists( Pstvs, Ngtvs, Xmax/Ymax/Xpad/Ypad ).

mk_pn_versus_lists( [], [], _ ).
mk_pn_versus_lists( [H-Hpstv|Tpstvs], [H-Hngtv|Tngtvs], Max ) :-
	mk_pn_versus_list_1( Tpstvs, Tngtvs, H, Hpstv, Hngtv, Max ),
	mk_pn_versus_lists( Tpstvs, Tngtvs, Max ).

mk_pn_versus_list_1( [], [], _B, _Bpstv, _Bngtv, _Max ).
mk_pn_versus_list_1( [H-Hpstv|Tpstvs], [H-Hngtv|Tngtvs], B, Bpstv, Bngtv, Max ) :-
	atom_codes( H, HCs ),
	atom_codes( B, BCs ),
	longest_prefix( HCs, BCs, PfxCs ), 
	append( PfxCs, PsfHCs, HCs ),
	append( PfxCs, PsfBCs, BCs ),
	append( HCs, "_vs_", HVsCs ),
	append( HVsCs, PsfBCs, PfxVsCs ),
	append( PfxVsCs, ".similarity_pos", PstvSimVsCs ),
	append( PfxVsCs, ".similarity_neg", NgtvSimVsCs ),
	atom_codes( PstvSimVs, PstvSimVsCs ),
	atom_codes( NgtvSimVs, NgtvSimVsCs ),
	open( PstvSimVs, write, Out ),
	write_list_of_lists_to_spaced_line_per_skim( [Bpstv,Hpstv], Out ),
	close( Out ),
	open( NgtvSimVs, write, Out ),
	write_list_of_lists_to_spaced_line_per_skim( [Bngtv,Hngtv], Out ),
	close( Out ),
	append( PfxVsCs, "_sim_pn.eps", SimEpsCs ),
	atom_codes( SimEps, SimEpsCs ),
	versus_plot_title( PfxCs, PsfBCs, PsfHCs, SetsTitle ), 
	Max = Xmax/Ymax/Xpad/Ypad,
	XmaxLim is Xmax + Xpad,
	YmaxLim is Ymax + Ypad,
	Opts = [term('postscript eps monochrome'),output(SimEps),
	        with(points), x(-Xpad,XmaxLim),y(-Ypad,YmaxLim),
		   set('key off'), set(SetsTitle),diagonal(true)],
	mktemp( sim_pn_tmp_XXXXXX, GPlotTmp ),
	gnucomfile( GPlotTmp, Opts, [f(PstvSimVs,[pt(2)]),f(NgtvSimVs,[pt(6)])] ),
	% gnucomfile( GPlotTmp, Opts, [PstvSimVs,NgtvSimVs] ),
	atom_concat( 'gnuplot "', GPlotTmp, GPlotPfx ), 
	atom_concat( GPlotPfx, '"', GPlot ),
	sys_info( GPlot ),
	sleep( 2 ),
	delete_file( GPlotTmp ),
	mk_pn_versus_list_1( Tpstvs, Tngtvs, B, Bpstv, Bngtv, Max ).

mk_versus_list( [], _Max ).
mk_versus_list( [H|T], Max ) :-
	mk_versus_list_1( T, H, Max ),
	mk_versus_list( T, Max ).

mk_versus_list_1( [], _B, _Max ).
mk_versus_list_1( [H-HList|T], B-BList, Max ) :-
	atom_codes( H, HCs ),
	atom_codes( B, BCs ),
	longest_prefix( HCs, BCs, PfxCs ), 
	append( PfxCs, PsfHCs, HCs ),
	append( PfxCs, PsfBCs, BCs ),
	append( HCs, "_vs_", HVsCs ),
	append( HVsCs, PsfBCs, PfxVsCs ),
	append( PfxVsCs, ".similarity", VersusCs ),
	atom_codes( Versus, VersusCs ),
	open( Versus, write, Out ),
	write_list_of_lists_to_spaced_line_per_skim( [BList,HList], Out ),
	close( Out ),
	mktemp( sim_tmp_XXXXXX, GPlotTmp ),
	append( PfxVsCs, "_sim.eps", SimEpsCs ),
	atom_codes( SimEps, SimEpsCs ),
	versus_plot_title( PfxCs, PsfBCs, PsfHCs, SetsTitle ), 
	Max = Xmax/Ymax,
	XmaxLim is Xmax + 0.2,
	YmaxLim is Ymax + 0.2,
	Opts = [term('postscript eps monochrome'),output(SimEps),
	        with(points), x(-0.2,XmaxLim),y(-0.2,YmaxLim),
		   set('key off'), set(SetsTitle),diagonal(true)],
	gnucomfile( GPlotTmp, Opts, Versus ),
	atom_concat( 'gnuplot "', GPlotTmp, GPlotPfx ), 
	atom_concat( GPlotPfx, '"', GPlot ),
	sys_info( GPlot ),
	sleep( 2 ),
	delete_file( GPlotTmp ),
	mk_versus_list_1( T, B-BList, Max ).

files_n_first_feature_prediction_pairs( [], _Ext, [] ).
files_n_first_feature_prediction_pairs( [H|T], Ext, [H-List|R] ) :-
	atom_concat( H, Ext, PctFile ),
	% atom_concat( H, '.predicted', PctFile ),
	get_first_v_with_agreeing_k_from_list_from_file( PctFile, List ),
	files_n_first_feature_prediction_pairs( T, Ext, R ).

get_first_v_with_agreeing_k_from_list_from_file( PctFile, List ) :-
	open( PctFile, read, In ),
	read( In, Distro ),
	get_first_v_with_agreeing_k_from_list_on_stream( Distro, _K, In, List ),
	close( In ).

versus_plot_title( PfxCs, PsfBCs, PsfHCs, SetsTitle ) :-
	flatten( ["title ",0'",0'(,PfxCs,") ",PsfBCs," vs. ",PsfHCs,0'"], FlatCs ),
	atom_codes( SetsTitle, FlatCs ).

get_first_v_with_agreeing_k_from_list_on_stream( end_of_file, _K, _In, List ) :-
	!,
	List = [].
get_first_v_with_agreeing_k_from_list_on_stream( [K-V|_], K, In, [V|R] ) :-
	!,
	read( In, NxtDistro ), 
	get_first_v_with_agreeing_k_from_list_on_stream( NxtDistro, K, In, R ).
get_first_v_with_agreeing_k_from_list_on_stream( Distro, K, In, List ) :-
	werr( [[something_doesnt_match(Distro,K,In,List)]] ),
	abort.

xy_opts( Opts, Xmax, Ymax, Xpad, Ypad ) :-
	( memberchk(xmax(XmaxAtm),Opts) ->
			to_number( XmaxAtm, Xmax ) ; Xmax is 1 ),
	( memberchk(ymax(YmaxAtm),Opts) ->
			to_number( YmaxAtm, Ymax ) ; Ymax is 1 ),
	( memberchk(xpad(XpadAtm),Opts) ->
			to_number( XpadAtm, Xpad ) ; Xpad is 0.2 ),
	( memberchk(xpad(YpadAtm),Opts) ->
			to_number( YpadAtm, Ypad ) ; Ypad is 0.2 ).
