
:- ensure_loaded( library(lists) ).					% memberchk/2.
:- ensure_loaded( '../../src/init_lib' ). % requires/1

:- ensure_loaded( '../src/cart_classifies' ).	% /3.
							               % distribution_to_correct/4.

:- requires( opts_have_help/1 ).
:- requires( opts_select_infiles/3 ).
:- requires( write_list_elem_per_line/1 ).
:- requires( write_list_elem_per_line_w_period/1 ).
:- requires( select_n_uniformly/5 ).
:- requires( read_terms/2 ).
:- requires( kv_decompose_on_max_v/3 ).
:- requires( kv_multi_vs_by/3 ).
:- requires( kv_div_vs_by/3 ).
:- requires( kv_sum_vs/2 ).
:- requires( kv_decompose/3 ).
:- requires( write_terms/2 ).

% upsh ~/islp/mcmcms/carts/auxil/test_classification train=pima_train test=pima_test method=ratio *mvisits 
% upsh ~/islp/mcmcms/carts/auxil/test_classification train=pima_train test=pima_test mvisits=tr_uc_rm_pima_idsd_train_a0_95b0_8_i50__s803.mvisits
% for fig.5 (ijcai05).
% upsh ~/islp/mcmcms/carts/auxil/test_classification train=pima_train test=pima_test mvisits=*mvisits method=bin

test_classification( Opts ) :-
	opts_have_help( Opts ),
	!,
	write( 'test_classification:     classify a number of test data points,' ),
	nl,
	write( '                         according to a training data-set and an' ),
	nl, 
	write( '                         mvisits file of an MCMC run.' ), nl, nl,
	write( '  Options.' ), nl,
	write( '    cohese(C)            cohesion method: {best,*distro*}' ), nl,
	write( '    file(M)              the mvisits files to process' ), nl,
	write( '    output(O)            output method: {bin,*pbc*,pbc_kv}' ), nl,
	write( '    sfx(Sfx)             suffix for extension of each file' ), nl,
	write( '    term(B)              term period at true' ), nl,
	write( '    test(S)              test data-set file' ), nl,
	write( '    train(N)             the training data-set.' ), nl,
	nl.

test_classification( Opts ) :-
	memberchk( train(Train), Opts ), 
	train_loaded_and_calls( Train, TrainData ),
	% findall( Mvs, member(mvisits(Mvs),Opts), MVsFs ), 
	opts_select_infiles( Opts, MVsFs, RmOpts ),
	write( files(MVsFs) ), nl,
	test_classification_1( MVsFs, TrainData, RmOpts ).

test_classification_1( [], _TrnD, _Opts ).
test_classification_1( [Mvs|T], TrainData, Opts ) :-
	write( doing(Mvs) ), nl,
	memberchk( test(Test), Opts ), 
	( memberchk( term(TBoo), Opts ) -> true; TBoo = true ),
	test_loaded( Test, TestData ),
	open( Mvs, read, In ), 
	read( In, InTerm ),
	stream_mvis_terms_to_classified( InTerm, TrainData, TestData, 0, In, Distros, ITms ),
	close( In ),
	write( user_error, done_classified ), nl( user_error ),
	data_to_true_classes( TestData, TrueClasses ), % HERE
	( memberchk(cohese(CohM),Opts) -> true; CohM = distro ), % alt. Meth= best
	( memberchk(output(OutM),Opts) -> true; OutM = pbc ), % alt. = bin, pbc_kv
	( memberchk( sfx(AllSfxPrv), Opts ) -> 
		( atom_concat( '_', _, AllSfxPrv ) ->
			AllSfxPrv = AllSfx
			;
			atom_concat( '_', AllSfxPrv, AllSfx )
		)
		; AllSfx = '' ),
	cohese_class_pairs( Distros, TrueClasses, CohM, OutM, ITms, Cohesed ), 
	% kv_decompose( CohesedPrs, Cohesed, _ ),
	atom_concat( Stem, '.mvisits', Mvs ),
	( (CohM == distro, (OutM == pbc; OutM == pbc_kv)) ->
		Sfx = '.predicted'
		;
		atom_concat( '.predicted_', CohM, Sfx1 ),
		atom_concat( '_', OutM, Sfx2 ),
		atom_concat( Sfx1, Sfx2, Sfx )
	),
	atom_concat( Stem, Sfx, TstClsPrv ),
	atom_concat( TstClsPrv, AllSfx, TstCls ),
	to_file_period_opt( TstCls, TBoo, Cohesed ),
	% atom_concat( '.predicted', MethsId, Sfx ),
	atom_concat( '.accuracy_bin', MethsId, AcrBinSfx ),
	atom_concat( '.accuracy_exp', MethsId, AcrExpSfx ),
	atom_concat( Stem, AcrBinSfx, AcrBinPrv ),
	atom_concat( AcrBinPrv, AllSfx, AcrBin ),
	atom_concat( Stem, AcrExpSfx, AcrExpPrv ),
	atom_concat( AcrExpPrv, AllSfx, AcrExp ),
	TestData = [HDatum|_],
	functor( HDatum, _, DAr ),
	count_accuracy( Cohesed, TestData, DAr, bin, 0, 0, BinAcrTrm ),
	count_accuracy( Cohesed, TestData, DAr, exp, 0, 0, ExpAcrTrm ),
	% count_accuracy( TrueTuples, bin, 0, 0, BinAcrTrm ),
	% count_accuracy( TrueTuples, exp, 0, 0, ExpAcrTrm ),
	write_terms( AcrBin, [BinAcrTrm] ),
	write_terms( AcrExp, [ExpAcrTrm] ),
	test_classification_1( T, TrainData, Opts ).

% count_accuracy( Cohesed, Data, DAr, Whc, AccSum, AccTot, AcrTrm ) :-
count_accuracy( [], _Data, _DAr, _Whc, AccSum, AccTot, AcrTrm ) :-
	AcrTrm = AccSum/AccTot.
count_accuracy( [HDst|TDsts], [HDtm|TDt], DAr, Whc, AccS, AccT, Acr ) :-
	arg( DAr, HDtm, TrueClass ),
	memberchk( TrueClass-TruePrb, HDst ),
	distribution_to_correct( HDst, TruePrb, TrueClass, Corr ),
	( Whc == bin ->
		( Corr == true ->
			NxSum is AccS + 1
			;
			NxSum is AccS
		)
		;
		NxSum is AccS + TruePrb
	),
	NxTot is AccT + 1,
	count_accuracy( TDsts, TDt, DAr, Whc, NxSum, NxTot, Acr ).

stream_mvis_terms_to_classified( end_of_file, _TrnD, _TstD, Tot, _In, [], Tot ) :- !.
stream_mvis_terms_to_classified( Tms-c(Cart), TrainData, TestData, Acc, In, [Distros-Tms|TPairs], Tot ) :-
	cart_classifies( Cart, TrainData, TestData, bayes, Distros, _ ),
	write( user_error, '.' ), nl( user_error ),
	!,
	NxAcc is Acc + Tms,
	read( In, InTerm ),
	stream_mvis_terms_to_classified( InTerm, TrainData, TestData, NxAcc, In, TPairs, Tot ).

cohese_class_pairs( [[]-_|_], _Empty, _CohM, _OutM, _Its, [] ) :- !.
cohese_class_pairs( Pairs, [HCls|TTC], CohM, OutM, Its, [ThisRec|Rest] ) :-
	kv_skim_ks( Pairs, Froth, Rem ),
	(CohM == best -> 
		distros_to_best_classes( Froth,  Best ),
		kv_on_k_add_vs( Best, SummedUnoInc ),
		Froth = [HFro-_|_],
		kv_adds_def_v_where_missing( HFro, SummedUnoInc, 0, SummedUno ),
		keysort( SummedUno, Summed ),
		( OutM== bin -> 
			kv_decompose_on_max_v( Summed, MaxClasses, _MaxV ),
			length( MaxClasses, Length ),
			select_n_uniformly( 1, MaxClasses, Length, [ThisRec], _ )
			% select_n_uniformly( 1, MaxClasses, Length, [ThisClass], _ ),
			% method_to_record_point( Meth, ThisClass, MaxV, Its, ThisRec ),
			;
			kv_sum_vs( Summed, VsSum ),
			kv_div_vs_by( Summed, VsSum, ThisRec )
		)
		;
		Froth = [HFro-Tms|TFro],
		kv_multi_vs_by( HFro, Tms, AdjHFro ),
		cohese_datum_distros( TFro, AdjHFro, CohDistro ),
		( OutM == bin ->
			kv_decompose_on_max_v( CohDistro, MaxClasses, _MaxV ),
			length( MaxClasses, Length ),
			select_n_uniformly( 1, MaxClasses, Length, [ThisRec], _ )
			;
			kv_sum_vs( CohDistro, VsSum ),
			kv_div_vs_by( CohDistro, VsSum, ThisRecPbc ),
			( OutM == pbc ->
				ThisRec = ThisRecPbc
				;
				OutM == pbc_kv,
				ThisRec = ThisRecPbc-HCls
			)
		)
	),
	cohese_class_pairs( Rem, TTC, CohM, OutM, Its, Rest ).

% cohese_datum_distros( +Pairs, +AccDistro, -Distro ) :-
% Distro is unormalised.
%
cohese_datum_distros( [], Distro, Distro ).
cohese_datum_distros( [H-Tms|T], AccDistro, Distro ) :-
	kv_multi_vs_by( H, Tms, Contr ),
	add_vs( AccDistro, Contr, NxDistro ),
	cohese_datum_distros( T, NxDistro, Distro ).

method_to_record_point( class, Class, _MaxTms, _Its, Class ).
method_to_record_point( ratio, _Class, MaxTms, Its, MaxTms/Its ).
method_to_record_point( both, Class, MaxTms, Its, Class:MaxTms/Its ).

kv_adds_def_v_where_missing( [], KVs, _Def, KVs ).
kv_adds_def_v_where_missing( [K-_V|T], AccKVs, Def, KVs ) :-
	( memberchk(K-_V2,AccKVs) ->
		AccKVs = NxAccKVs
		;
		NxAccKVs = [K-Def|AccKVs]
	),
	kv_adds_def_v_where_missing( T, NxAccKVs, Def, KVs ).


kv_on_k_add_vs( [], [] ).
kv_on_k_add_vs( [Hk-Hv|T], Summed ) :-
	( select(Hk-OthV,T,Rem) ->
		NxV is OthV + Hv,
		NexT = [Hk-NxV|Rem],
		TSummed = Summed
		;
		NexT = T,
		Summed = [Hk-Hv|TSummed]
	),
	kv_on_k_add_vs( NexT, TSummed ).

kv_skim_ks( [], [], [] ).
kv_skim_ks( [[H|TKs]-V|TPrs], [H-V|Rst], [TKs-V|TRem] ) :-
	kv_skim_ks( TPrs, Rst, TRem ).

add_vs( [], [], [] ).
add_vs( [K-V1|T1], [K-V2|T2], [K-Sum|TSums] ) :-
	Sum is V1 + V2,
	add_vs( T1, T2, TSums ).

distros_to_best_classes( [],  [] ).
distros_to_best_classes( [H-Tms|T],  [HBst-Tms|TBst] ) :-
	kv_decompose_on_max_v( H, MaxKs, _MaxV ),
	length( MaxKs, Length ),
	select_n_uniformly( 1, MaxKs, Length, [HBst], _ ),
	distros_to_best_classes( T, TBst ).

train_loaded_and_calls( pima_train, TrainData ) :-
	ensure_loaded( '$HOME/islp/mcmcms/carts/pima/data/pima_idsd_train.pl' ), 
	NofF is 8,
	pima_idsd_train( _Splits, _Ids ),
	nof_features_to_train_data( NofF, TrainData ).
train_loaded_and_calls( icml_pima, TrainData ) :-
	ensure_loaded( '$HOME/islp/mcmcms/pbl/icml05/data/idsd' ), 
	NofF is 8,
	idsd( NofF, pima_train_080, _Splits, _Ids ),
	nof_features_to_train_data( NofF, TrainData ).
train_loaded_and_calls( icml_bcw, TrainData ) :-
	ensure_loaded( '$HOME/islp/mcmcms/pbl/icml05/data/idsd' ), 
	NofF is 9,
	idsd( NofF, bcw_train_080, _Splits, _Ids ),
	nof_features_to_train_data( NofF, TrainData ).
train_loaded_and_calls( icml_kyph, TrainData ) :-
	ensure_loaded( '$HOME/islp/mcmcms/pbl/icml05/data/idsd' ), 
	NofF is 3,
	idsd( NofF, kyph_train_080, _Splits, _Ids ),
	nof_features_to_train_data( NofF, TrainData ).
train_loaded_and_calls( icml_letrec, TrainData ) :-
	ensure_loaded( '$HOME/islp/mcmcms/pbl/icml05/data/idsd' ), 
	NofF is 16,
	idsd( NofF, letter_recognition_train_080, _Splits, _Ids ),
	nof_features_to_train_data( NofF, TrainData ).
train_loaded_and_calls( pima_ijcai05_synth_500, TrainData ) :-
	ensure_loaded( '$HOME/islp/mcmcms/pbl/ijcai05/data/idsd' ), 
	idsd( 8, sg500, _Splits, _Ids ),
	findall( [A,B,C,D,E,F,G,H]-I, data(A,B,C,D,E,F,G,H,I), TrainData ).

nof_features_to_train_data( NofF, TrainData ) :-
	NofArgs is NofF + 1,
	functor( Call, data, NofArgs ),
	data_call_and_feature_list( NofF, NofArgs, Call, FList, Kat ),
	findall( FList-Kat, Call, TrainData ).

test_loaded( pima_test, TestData ) :-
	!,
	read_terms( '$HOME/islp/mcmcms/carts/pima/data/pima_test.pl', TestData ).
test_loaded( File, TestData ) :-
	read_terms( File, TestData ).

to_file_period_opt( TstCls, TBoo, Cohesed ) :-
	tell( TstCls ),
	( TBoo == false ->
		write_list_elem_per_line( Cohesed )
		;
		write_list_elem_per_line_w_period( Cohesed )
	),
	told.

data_to_true_classes( [H|T], [Fst|Rst] ) :-
	functor( H, data, NofArgs ),
	arg( NofArgs, H, Fst ),
	data_to_true_classes_1( T, NofArgs, Rst ).

data_to_true_classes_1( [], _NofArgs, [] ).
data_to_true_classes_1( [H|T], NofArgs, [Fst|Rst] ) :-
	arg( NofArgs, H, Fst ),
	data_to_true_classes_1( T, NofArgs, Rst ).
