:- ensure_loaded( joint_prob  ). % for joint_prob


/*---------------------------------------------------------------------
  Predicate: model_llhood(+Model,-LogLikelihood)
       Args: 
   Succeeds: exactly once
   Comments: Collect up markers, then compute llhood
---------------------------------------------------------------------*/
model_llhood(Model,LLH) :-
	findall(Marker,allele_system(Marker,_,_,_),Markers),
	model_llhood(Markers,Model,0,LLH).

/*---------------------------------------------------------------------
  Predicate: model_llhood(+Markers,+Model,+In,-Prob)
       Args: 
   Succeeds: exactly once
   Comments: Plod thro' makers adding log probabilities as we go
---------------------------------------------------------------------*/
model_llhood([],_Model,LLH,LLH).
model_llhood([Marker|T],Model,LLHIn,LLHOut) :-
	model_marker_prob(Model,Marker,Prob),
	(
	  Prob = 0.0                       % bomb out early if 
	->                                 % inconsistent
	  % LLHMid is log(0.0),
	  LLHMid is -inf,
	  Rem=[]
	;
	  LLHMid is LLHIn + log(Prob),
	  Rem=T
	),		
	model_llhood(Rem,Model,LLHMid,LLHOut).
/*---------------------------------------------------------------------
  Predicate: model_marker_prob(+Model,+Marker,-Prob)
       Args: 
   Succeeds: exactly once
   Comments: Grab case data and then compute likelihood
---------------------------------------------------------------------*/
model_marker_prob(Model,Marker,Prob) :-
	findall(
		Person-(Marker-alleles(A1,A2)),
		(
		  case_data(Person,Marker,A1,A2),   % grab data
		  A1 @=< A2                         % should always succeed
		),
		Inst
	       ),
	joint_prob(Inst,Model,Prob).                % defined elsewhere 


	  























%/*---------------------------------------------------------------------
%  Predicate: inherit_prob(+ChildAlleles,+InheritedMotherAllele,+InheritedFatherAllele,-Prob).
%       Args: 
%   Succeeds: exactly once
%   Comments: simple matching
%---------------------------------------------------------------------*/
%inherit_prob(alleles(A1,A2),A1,A2,1.0) :- !.
%inherit_prob(alleles(A1,A2),A2,A1,1.0) :- !.
%inherit_prob(alleles(_A1,_A2),_MA,_FA,0.0).
	

%/*---------------------------------------------------------------------
%  Predicate: half_prob(+ChildAlleles,+InheritedParentAllele,-Prob)
%       Args: 
%   Succeeds: exactly once
%   Comments: 
%---------------------------------------------------------------------*/
%half_prob(Marker-alleles(A1,A2),PA,Prob) :-
%	(
%	  select(PA,[A1,A2],[Other])    %If PA is there
%	->
%	  allele_system(Marker,Alleles),%get background prob of other one
%	  member(Other-Prob,Alleles)
%	;
%	  Prob = 0.0		        %else zero probability
%	).

% no named parents, homozygous
% mutation is irrelevant
%cpt(_Child-(Marker-alleles(A1,A1)),[]-[],Prob) :-
%	!,      % green cut
%	allele_system(Marker,Alleles),
%	memberchk(A1-P1,Alleles),
%	Prob is P1*P1.

%% no named parents, heterozygous
%% mutation is irrelevant
%cpt(_Child-(Marker-alleles(A1,A2)),[]-[],Prob) :-
%	!,     % needs cut above
%	A1 @< A2,
%	allele_system(Marker,Alleles),
%	memberchk(A1-P1,Alleles),
%	memberchk(A2-P2,Alleles),
%	Prob is 2*P1*P2.

%% only one named parent, homozygous
%% TODO
%cpt(_Child-(Marker-alleles(A1,A1)),[_Parent]-[Marker-alleles(PA1,PA2)],Prob) :-
%	half_prob(Marker-alleles(A1,A2),PA1,Prob1),
%	half_prob(Marker-alleles(A1,A2),PA2,Prob2),
%	Prob is 0.5*Prob1 + 0.5*Prob2.

%% only one named parent, heterozygous
%% TODO
%cpt(_Child-(Marker-alleles(A1,A2)),[_Parent]-[Marker-alleles(PA1,PA2)],Prob) :-
%	half_prob(Marker-alleles(A1,A2),PA1,Prob1),
%	half_prob(Marker-alleles(A1,A2),PA2,Prob2),
%	Prob is 0.5*Prob1 + 0.5*Prob2.

%% two named parents, homozygous
%cpt(_Child-(Marker-alleles(A1,A1)),[_Mother,_Father]-[Marker-alleles(MA1,MA2),Marker-alleles(FA1,FA2)],Prob) :-
%	paternal_maternal_inherit_prob(Marker,A1,alleles(MA1,MA2),MaternalProb),
%	paternal_maternal_inherit_prob(Marker,A1,alleles(FA1,FA2),PaternalProb),
%	Prob is MaternalProb * PaternalProb.

%% two named parents, heterozygous
%cpt(_Child-(_Marker-alleles(A1,A2)),[_Mother,_Father]-[_Marker-alleles(MA1,MA2),_Marker-alleles(FA1,FA2)],Prob) :-
%	A1 @< A2,
%	inherit_prob(alleles(A1,A2),MA1,FA1,Prob11),
%	inherit_prob(alleles(A1,A2),MA1,FA2,Prob12),
%	inherit_prob(alleles(A1,A2),MA2,FA1,Prob21),
%	inherit_prob(alleles(A1,A2),MA2,FA2,Prob22),
%	Prob is 0.25*Prob11 + 0.25*Prob12 + 0.25*Prob21 + 0.25*Prob22.

%/*---------------------------------------------------------------------
%  Predicate: get_poss_alleles(+Persons,+Marker,+PossAllelesIn,-PossAllelesOut)
%       Args: 
%   Succeeds: exactly once
%   Comments: Collect possible alleles for each person (with accumulator)
%---------------------------------------------------------------------*/
%get_poss_alleles([],_Marker,PossAlleles,PossAlleles).
%get_poss_alleles([Person|T],Marker,In,Out) :-
%	get_poss_alleles_person(Person,Marker,PersonPossAlleles),
%	get_poss_alleles(T,Marker,[Person-PersonPossAlleles|In],Out).


%go :-
	
%/*---------------------------------------------------------------------
%  Predicate: get_poss_alleles_person(+Person,+Marker,-PossAlleles)
%       Args: 
%   Succeeds: exactly once
%   Comments: Only two possibilities:
%             we know exactly the alleles for this person and marker
%	     or all alleles in the system are possible
%	     (returning a massive list)!
%---------------------------------------------------------------------*/
%get_poss_alleles_person(Person,Marker,[Marker-alleles(A1,A2)]) :-
%	case_data(Person,Marker,A1,A2),
%	!,
%	A1 @=< A2.      % have to be in order in the case data file
%%get_poss_alleles_person(_Person,Marker,[Marker-'$all']).
%get_poss_alleles_person(_Person,Marker,Alleles) :-
%	allele_system(Marker,AFs,_,_),
%	findall(
%		Marker-alleles(A1,A2),
%		(
%		  member(A1-_P1,AFs),
%		  member(A2-_P2,AFs),
%		  A1 @=< A2
%		),
%		Alleles
%	       ).

% assume that if there are the right number of vals, then they are all there!
%all_vals(_Var,Vals) :-                  % same for all persons
%	Vals = [Marker-_|_],            % find the marker
%	allele_system(Marker,Alleles,_,_),  % alleles for this marker
%	length(Alleles,NumAlleles),     % how many
%	NumPossAlleles is NumAlleles * NumAlleles, %how many possible
% 	length(Vals,NumPossAlleles).               %all there
